@@ -68,7 +68,8 @@ def interpret_line(message):
 			line_return = script_var(g.xgrid, g.ygrid, g.zgrid,
 						 [["\""+message[start_tilde+1:end_tilde]+"\"", 1]])
 			#Add the returned data into the end string.
-			if type(line_return) == str: line_return = line_return[1:-1]
+			if type(line_return) == str or type(line_return) == unicode:
+				line_return = line_return[1:-1]
 			endstring += str(line_return)
 		cur_str_pos = end_tilde
 	return endstring
@@ -176,7 +177,7 @@ def match_parenth(command):
 			else:
 				par_num -= 1
 		i += 1
-	print "match_parenth()  cannot find the end of this parenthesis pair:"
+	print "match_parenth() cannot find the end of this parenthesis pair:"
 	print command
 	return -1
 
@@ -321,6 +322,9 @@ def run_command(x, y, z, command):
 	elif switch == "equip":  #adjust equipment
 		if check_num_args(len(arg_list), 2, 2, "equip"):
 			return script_equip(x, y, z, arg_list)
+	elif switch  == "fade":
+		if check_num_args(len(arg_list), 1, 1, "fade"):
+			return script_fade(x, y, z, arg_list)
 	elif switch  == "find":
 		if check_num_args(len(arg_list), 2, 2, "find"):
 			return script_find(x, y, z, arg_list)
@@ -452,6 +456,7 @@ def check_num_args(num_of_args, lower_num, upper_num, command_name):
 #function is called.
 def check_types_args(args, arg_types, command_name, quiet=0):
 	for i in range(len(arg_types)):
+		if args[i] == "bad": return 0
 		if args[i][1] != arg_types[i] and arg_types[i] != 2:
 			if quiet == 0:
 				print command_name + " called with wrong type of arguments:"
@@ -470,6 +475,10 @@ def script_addpix(x, y, z, argument_array): #add a picture to the tile.
 	if check_types_args(argument_array, [1], "addpix") == 0:
 		return "bad"
 
+	if not g.tiles.has_key(argument_array[0][0][1:-1]):
+		print "Tile "+argument_array[0][0][1:-1]+" does not exist in map "+\
+			g.maps[z].name
+		return 0
 	g.maps[z].field[y][x].add_pix(g.tiles[argument_array[0][0][1:-1]])
 	return 1
 
@@ -513,12 +522,14 @@ def script_attack(x, y, z, argument_array):  #cause creature to attack
 			print "monster " + temp_name + " not found"
 	#if there exists a monster to battle:
 	if temp != -1:
+		main.key_down = [False, False, False, False]
 		return main.start_battle(temp)
 	return 0
 
 def script_damage_monster(x, y, z, argument_array):  #In battle, hurt monster for either
 		#adj_attack points, or Command points. *NOT* reduced by armor.
-	if g.cur_window != "battle":
+	if (g.cur_window != "battle" and g.cur_window != "battle_item" and
+		g.cur_window != "battle_skill"):
 		print "damage_monster called outside of battle."
 		return -1
 
@@ -540,7 +551,7 @@ def script_delpix(x, y, z, argument_array): #remove a picture from the tile.
 
 	try:
 		g.maps[z].field[y][x].del_pix(g.tiles[argument_array[0][0][1:-1]])
-		main.refreshmap()
+		main.refresh_tile(x, y, z)
 		return 1
 	except ValueError:
 		return 0
@@ -603,6 +614,22 @@ def script_equip(x, y, z, argument_array): #adjust equipment
 		print "unknown switch for equip(): "+switch2
 		return "bad"
 
+def script_fade(x, y, z, argument_array):
+	if check_types_args(argument_array, [0], "fade") == 0:
+		return "bad"
+
+	if argument_array[0][1] == 0:
+		for i in range(40):
+			g.pygame.time.wait(15)
+			g.screen.fill(g.colors["black"], (0, i*12, g.screen_size[0], 12))
+			g.screen.fill(g.colors["black"], (0, g.screen_size[1]-i*12-12,
+				g.screen_size[0], 12))
+			g.screen.fill(g.colors["black"], (0, g.screen_size[1]/2+i*12,
+				g.screen_size[0], 12))
+			g.screen.fill(g.colors["black"], (0, g.screen_size[1]/2-i*12-12,
+				g.screen_size[0], 12))
+			g.pygame.display.flip()
+
 def script_find(x, y, z, argument_array):
 	if check_types_args(argument_array, [1, 2], "find") == 0:
 		return "bad"
@@ -686,7 +713,8 @@ def script_give(x, y, z, argument_array):  #Change stats. Note that negative num
 
 		#note that name is not interpreted as a command.
 		player.name = interpret_line(argument_array[1][0][1:-1])
-		main.recalc_stats()
+		main.refresh_bars()
+		main.refresh_inv_icon()
 		return 1
 
 	else:
@@ -708,7 +736,8 @@ def script_give(x, y, z, argument_array):  #Change stats. Note that negative num
 	elif switch2 == "exp": player.add_exp(set_to)
 	elif switch2 == "skillpoints": player.give_stat("skillpoints", set_to)
 	else: print "Unknown stat: " + switch2
-	main.recalc_stats()
+	main.refresh_bars()
+	main.refresh_inv_icon()
 	if set_to == 0: return 0
 	else: return 1
 
@@ -732,7 +761,8 @@ def script_hurt(x, y, z, argument_array):  #hurt player (reduced by armor)
 
 def script_hurt_monster(x, y, z, argument_array):  #In battle, hurt monster for either
 		#adj_attack points, or Command points. Reduced by armor.
-	if g.cur_window != "battle":
+	if (g.cur_window != "battle" and g.cur_window != "battle_item" and
+		g.cur_window != "battle_skill"):
 		print "hurt_monster called outside of battle."
 		return -1
 
@@ -832,9 +862,11 @@ def script_inv(x, y, z, argument_array): #inventory functions.
 							and tmptype != 16 and tmptype != 17):
 			print "item " + g.item.item[temp].name + " cannot be used."
 			return "bad"
-		if g.cur_window == "battle" and (tmptype != 16):
+		if (g.cur_window == "battle" or g.cur_window == "battle_item" or
+		g.cur_window == "battle_skill") and (tmptype != 16):
 			battle.useitem(temp, 1)
-		elif g.cur_window != "battle" and (tmptype == 11 or tmptype == 16 or
+		elif (g.cur_window != "battle" and g.cur_window != "battle_item" and
+		g.cur_window != "battle_skill") and (tmptype == 11 or tmptype == 16 or
 							tmptype == 17):
 			main.inv.use_item(temp)
 			return 1
@@ -890,7 +922,7 @@ def script_is_equipped(x, y, z, argument_array): #is the given item equipped?
 
 def script_item(x, y, z, argument_array):  #give item
 	argument_array = [["\"give\"", 1], argument_array[0]]
-	return script_inv(x, y, z, argument_array)
+	tmp= script_inv(x, y, z, argument_array)
 	if check_types_args(argument_array, [1], "item") == 0:
 		return "bad"
 
@@ -977,14 +1009,15 @@ def script_mapstat(x, y, z, argument_array):
 	elif switch2 == "change_titlebar":
 		switch3 = interpret_line(switch3)
 		g.game_name = switch3
-		g.window_main.title(g.game_name)
+		g.pygame.display.set_caption(g.game_name)
 		return 1
 
 #In battle, change the stats of the given monster
 def script_monster_give_stat(x, y, z, argument_array):
 	if check_types_args(argument_array, [2, 1, 2], "monster_give_stat") == 0:
 		return "bad"
-	if g.cur_window != "battle":
+	if (g.cur_window != "battle" and g.cur_window != "battle_item" and
+		g.cur_window != "battle_skill"):
 		print "monster_stat called outside of battle."
 		return -1
 
@@ -1028,7 +1061,8 @@ def script_monster_give_stat(x, y, z, argument_array):
 def script_monster_stat(x, y, z, argument_array):
 	if check_types_args(argument_array, [0, 1], "monster_stat") == 0:
 		return "bad"
-	if g.cur_window != "battle":
+	if (g.cur_window != "battle" and g.cur_window != "battle_item" and
+		g.cur_window != "battle_skill"):
 		print "monster_stat called outside of battle."
 		return -1
 
@@ -1060,7 +1094,8 @@ def script_monster_stat(x, y, z, argument_array):
 
 def script_monster_select(x, y, z, argument_array):  #In battle, ask the player to
 									#select a monster
-	if g.cur_window != "battle":
+	if (g.cur_window != "battle" and g.cur_window != "battle_item" and
+		g.cur_window != "battle_skill"):
 		print "monster_select called outside of battle."
 		return -1
 
@@ -1243,7 +1278,8 @@ def script_skill(x, y, z, argument_array):  #skill functions
 			g.player.skill[temp][5] = 1
 			return 1
 	elif argument_array[0][0][1:-1].lower() == "use":
-		if g.cur_window != "battle":
+		if (g.cur_window != "battle" and g.cur_window != "battle_item" and
+				g.cur_window != "battle_skill"):
 			print "Cannot use skills outside of battle."
 			return "bad"
 		battle.useskill(temp, 1)
