@@ -19,13 +19,15 @@
 #This file contains everything that needs to be accessed by most or all files.
 
 
-#Dragon Hunt 3.51. Released under the GPL. Copyright 2005
+#Dragon Hunt 3.55. Released under the GPL. Copyright 2005
 #(Note to self: when changing version, change variables.txt, README.txt,
 # and setup.py.)
 
-from Tkinter import *
-import ImageTk
+#from Tkinter import *
+#import ImageTk
 #needed for die_roll
+import pygame
+
 from random import random
 #needed for save/load game
 import pickle
@@ -49,6 +51,9 @@ game_name = ""
 #a Tkinter window reference later.)
 window_main = ""
 
+#Call rpg with the debug argument to show debug info.
+debug = False
+
 #fill colour
 fill_colour = "#b9e5ad"
 fill_sel_colour = "#7aa96e"
@@ -97,17 +102,8 @@ mod_directory = ""
 per_turn_script = []
 
 #Default key bindings for the game.
-binding_up="<Up>"
-binding_down="<Down>"
-binding_left="<Left>"
-binding_right="<Right>"
-binding_action="<Return>"
-binding_cancel="<Escape>"
-binding_attack="a"
-binding_save="s"
-binding_quit="q"
-binding_inv="i"
-binding_console="~"
+bindings = {}
+
 message_lines = 6
 difficulty = 1
 
@@ -120,12 +116,21 @@ gold_name = "Gold"
 skill_name = "Skill"
 exp_name = "EXP"
 level_name = "Level"
-
+exp_list = ""
 
 
 #Used in new_game
 default_player_name = "Alfred"
 
+#Used in place of some of the Tkinter variables.
+break_one_loop = 0
+
+#used to decide whether or not to refresh (flip) the screen.
+unclean_screen = False
+
+global clock
+clock = pygame.time.Clock()
+
 #save the game in saves/input.
 #uses pickle, first line is a version number.
 def savegame(save_file):
@@ -163,7 +168,7 @@ def savegame(save_file):
 		else: pickle.dump("Ignore", savefile)
 	pickle.dump(xgrid, savefile)
 	pickle.dump(ygrid, savefile)
-	pickle.dump(zgrid, savefile)
+	pickle.dump(g.maps[zgrid].name, savefile)
 	#skills are stored by name as well.
 	num = 0
 	for i in range(len(player.skill)):
@@ -211,6 +216,8 @@ def loadgame(save_file):
 	global xgrid;		xgrid = pickle.load(savefile)
 	global ygrid;		ygrid = pickle.load(savefile)
 	global zgrid;		zgrid = pickle.load(savefile)
+	if str(zgrid).isdigit() != True:
+		zgrid = mapname2zgrid(str(zgrid))
 	skill_len = pickle.load(savefile)
 	for i in range(skill_len):
 		player.skill[findskill(pickle.load(savefile))][5] = 1
@@ -261,19 +268,60 @@ def loadgame_31(save_file):
 
 #this calls scripting.py to read the datafiles.
 def init_data():
+	screen.fill(colors["light_gray"], (screen_size[0]/2-150, screen_size[1]/2-20,
+		300, 40))
+	print_string(screen, "Loading Settings", font,
+		(screen_size[0]/2, screen_size[1]/2), align=1)
+	pygame.display.flip()
 	read_settings()
-	read_maps()
+	screen.fill(colors["light_gray"], (screen_size[0]/2-150, screen_size[1]/2-20,
+		300, 40))
+	print_string(screen, "Loading Backgrounds", font,
+		(screen_size[0]/2, screen_size[1]/2), align=1)
+	pygame.display.flip()
+	load_backgrounds()
+	screen.fill(colors["light_gray"], (screen_size[0]/2-150, screen_size[1]/2-20,
+		300, 40))
+	print_string(screen, "Loading Scripts", font,
+		(screen_size[0]/2, screen_size[1]/2), align=1)
+	pygame.display.flip()
 	read_scripts()
+	screen.fill(colors["light_gray"], (screen_size[0]/2-150, screen_size[1]/2-20,
+		300, 40))
+	print_string(screen, "Loading Items", font,
+		(screen_size[0]/2, screen_size[1]/2), align=1)
+	pygame.display.flip()
 	item.read_items()
+	screen.fill(colors["light_gray"], (screen_size[0]/2-150, screen_size[1]/2-20,
+		300, 40))
+	print_string(screen, "Loading Skills", font,
+		(screen_size[0]/2, screen_size[1]/2), align=1)
+	pygame.display.flip()
 	read_skills()
+	screen.fill(colors["light_gray"], (screen_size[0]/2-150, screen_size[1]/2-20,
+		300, 40))
+	print_string(screen, "Loading Monsters", font,
+		(screen_size[0]/2, screen_size[1]/2), align=1)
+	pygame.display.flip()
 	monster.read_monster()
 	global game_name
 	game_name = read_game_name()
 	read_variables()
+	screen.fill(colors["light_gray"], (screen_size[0]/2-150, screen_size[1]/2-20,
+		300, 40))
+	print_string(screen, "Loading Shops", font,
+		(screen_size[0]/2, screen_size[1]/2), align=1)
+	pygame.display.flip()
 	read_shops()
 	read_perturn()
+	screen.fill(colors["light_gray"], (screen_size[0]/2-150, screen_size[1]/2-20,
+		300, 40))
+	print_string(screen, "Loading Images", font,
+		(screen_size[0]/2, screen_size[1]/2), align=1)
+	pygame.display.flip()
 	load_buttons()
-        load_icons()
+	load_icons()
+	load_sounds()
 
 #Get the name of the module from the variables.txt file. Note that this
 #does not set the game name, only returns it. This lets it be used in rpg.py
@@ -295,19 +343,15 @@ new_game_dice = []
 
 #get the key bindings from settings.txt
 def read_settings():
-	global binding_up
-	global binding_down
-	global binding_left
-	global binding_right
-	global binding_action
-	global binding_cancel
-	global binding_attack
-	global binding_save
-	global binding_quit
-	global binding_inv
-	global binding_console
+	global bindings
 	global message_lines
 	global difficulty
+	global editor_xy
+	editor_xy = (640, 480)
+	global editor_tilesize
+	editor_tilesize = 15
+	global fullscreen
+	fullscreen = 0
 	#try to open settings.txt. If it doesn't exist,
 	# just use the default settings.
 	try: file = open("../settings.txt", 'r')
@@ -318,34 +362,23 @@ def read_settings():
 		if (line[:1] == "#" or line[:1] == ""):
 			line = file.readline()
 			continue
-		if (line.split("=", 1)[0].strip() == "up"):
-			binding_up = line.split("=", 1)[1].strip()
-		if (line.split("=", 1)[0].strip() == "down"):
-			binding_down = line.split("=", 1)[1].strip()
-		if (line.split("=", 1)[0].strip() == "left"):
-			binding_left = line.split("=", 1)[1].strip()
-		if (line.split("=", 1)[0].strip() == "right"):
-			binding_right = line.split("=", 1)[1].strip()
-		if (line.split("=", 1)[0].strip() == "action"):
-			binding_action = line.split("=", 1)[1].strip()
-		if (line.split("=", 1)[0].strip() == "cancel"):
-			binding_cancel = line.split("=", 1)[1].strip()
-		if (line.split("=", 1)[0].strip() == "attack"):
-			binding_attack = line.split("=", 1)[1].strip()
-		if (line.split("=", 1)[0].strip() == "save"):
-			binding_save = line.split("=", 1)[1].strip()
-		if (line.split("=", 1)[0].strip() == "quit"):
-			binding_quit = line.split("=", 1)[1].strip()
-		if (line.split("=", 1)[0].strip() == "inv"):
-			binding_inv = line.split("=", 1)[1].strip()
-		if (line.split("=", 1)[0].strip() == "load_console"):
-			binding_console = line.split("=", 1)[1].strip()
 		if (line.split("=", 1)[0].strip() == "message_lines"):
 			message_lines = line.split("=", 1)[1].strip()
-		if (line.split("=", 1)[0].strip() == "difficulty"):
+		elif (line.split("=", 1)[0].strip() == "difficulty"):
 			difficulty = int(line.split("=", 1)[1].strip())
+		elif (line.split("=", 1)[0].strip() == "editor_xsize"):
+			editor_xy = (int(line.split("=", 1)[1].strip()), editor_xy[1])
+		elif (line.split("=", 1)[0].strip() == "editor_ysize"):
+			editor_xy = (editor_xy[0], int(line.split("=", 1)[1].strip()))
+		elif (line.split("=", 1)[0].strip() == "editor_tilesize"):
+			editor_tilesize = int(line.split("=", 1)[1].strip())
+		elif (line.split("=", 1)[0].strip() == "fullscreen"):
+			fullscreen = int(line.split("=", 1)[1].strip())
+		else:
+			bind_line = line.split("=", 1)[0].strip()
+			bindings[bind_line] = int(line.split("=", 1)[1].strip())
 		line = file.readline()
-
+read_settings()
 
 def read_variables():
 	for i in range(5):
@@ -401,6 +434,9 @@ def read_variables():
 		elif (switch2 == "level_name"):
 			global level_name
 			level_name = line.split("=", 1)[1].strip()
+		elif (switch2 == "exp_list"):
+			global exp_list
+			exp_list = line.split("=", 1)[1].strip().split(" ")
 		line = file.readline()
 
 #given a dice set and string of the form 2d4+5, place in new_game_dice
@@ -443,7 +479,8 @@ def read_perturn():
 #skills array. Each skill is a separate line in the array. Each line goes:
 #name, effect, level, price, description, acquired, scripting, picture.
 #effect is an integer that tells battle.py which case in a select to pick.
-#(0=Rage, 1=Sneak, 2=Frenzy, 3=Dismember, 4=Scripted.)
+#(0=Rage, 1=Sneak, 2=Frenzy, 3=Dismember, 4=Scripted (battle),
+# 5=Scripted (out of battle), 6=Scripted (both).)
 #level is the skillpoints required to get the skill,
 #price is the ep needed to use.
 #acquired tells if the skill has already been learned by the player.
@@ -498,6 +535,7 @@ def read_skills():
 	#temp storage for the skill data
 	temp_skill_name = ""
 	temp_skill_level = 0
+	temp_skill_type = 4
 	temp_skill_price = 0
 	temp_skill_description = ""
 	temp_skill_scripting = []
@@ -515,12 +553,13 @@ def read_skills():
 			else: #Or just input a new skill.
 				#If we have any data, add the previous skill
 				if temp_skill_name != "":
-					addskill(temp_skill_name, 4, temp_skill_level,
+					addskill(temp_skill_name, temp_skill_type, temp_skill_level,
 							temp_skill_price, temp_skill_description,
 							temp_skill_scripting, temp_skill_picture)
 
 				temp_skill_name = line_strip[1:]
 				temp_skill_level = 0
+				temp_skill_type = 4
 				temp_skill_price = 0
 				temp_skill_description = ""
 				temp_skill_scripting = []
@@ -531,12 +570,13 @@ def read_skills():
 				value = line_strip.split("=", 1)[1].strip()
 				if command == "level": temp_skill_level = value
 				elif command == "price": temp_skill_price = value
+				elif command == "type": temp_skill_type = value
 				elif command == "description": temp_skill_description = value
 				elif command == "picture": temp_skill_picture = value
 			else:
 				temp_skill_scripting.append(line_strip)
 	if temp_skill_name != "":
-		addskill(temp_skill_name, 4, temp_skill_level,
+		addskill(temp_skill_name, temp_skill_type, temp_skill_level,
 				temp_skill_price, temp_skill_description,
 				temp_skill_scripting, temp_skill_picture)
 
@@ -575,15 +615,15 @@ def checklocation(x, y):
 
 #like checklocation, but just returns 0 (unwalkable) or 1 (walkable)
 def iswalkable(x, y, dx, dy):
-        #set direction hero is moving in
-        if dy == -1:
-            move_direction = "n"
-        elif dy == 1:
-            move_direction = "s"
-        elif dx == -1:
-            move_direction = "w"
-        else:
-            move_direction = "e"
+	#set direction hero is moving in
+	if dy == -1:
+		move_direction = "n"
+	elif dy == 1:
+		move_direction = "s"
+	elif dx == -1:
+		move_direction = "w"
+	else:
+		move_direction = "e"
 
 	#Assume that all off-map areas (to north or west) are unwalkable.
 	if x < 0 or y < 0:
@@ -592,25 +632,25 @@ def iswalkable(x, y, dx, dy):
 		#this will fail when looking too far right or down
 		if maps[zgrid].field[y][x].walk == 0:
 			return 0
-                #can't move onto tile if there's a wall in the way
-                if maps[zgrid].field[y][x].walk == 1:
-                    if maps[zgrid].field[y][x].wall_s == 1 and move_direction == "n":
-                        return 0
-                    elif maps[zgrid].field[y][x].wall_n == 1 and move_direction == "s":
-                        return 0
-                    elif maps[zgrid].field[y][x].wall_e == 1 and move_direction == "w":
-                        return 0
-                    elif maps[zgrid].field[y][x].wall_w == 1 and move_direction == "e":
-                        return 0
-                #can't move out of old tile if there's a wall in the way
-                if maps[zgrid].field[y-dy][x-dx].wall_s == 1 and move_direction == "s":
-                    return 0
-                elif maps[zgrid].field[y-dy][x-dx].wall_n == 1 and move_direction == "n":
-                    return 0
-                elif maps[zgrid].field[y-dy][x-dx].wall_e == 1 and move_direction == "e":
-                    return 0
-                elif maps[zgrid].field[y-dy][x-dx].wall_w == 1 and move_direction == "w":
-                    return 0
+		#can't move onto tile if there's a wall in the way
+		if maps[zgrid].field[y][x].walk == 1:
+			if maps[zgrid].field[y][x].wall_s == 1 and move_direction == "n":
+				return 0
+			elif maps[zgrid].field[y][x].wall_n == 1 and move_direction == "s":
+				return 0
+			elif maps[zgrid].field[y][x].wall_e == 1 and move_direction == "w":
+				return 0
+			elif maps[zgrid].field[y][x].wall_w == 1 and move_direction == "e":
+				return 0
+		#can't move out of old tile if there's a wall in the way
+		if maps[zgrid].field[y-dy][x-dx].wall_s == 1 and move_direction == "s":
+			return 0
+		elif maps[zgrid].field[y-dy][x-dx].wall_n == 1 and move_direction == "n":
+			return 0
+		elif maps[zgrid].field[y-dy][x-dx].wall_e == 1 and move_direction == "e":
+			return 0
+		elif maps[zgrid].field[y-dy][x-dx].wall_w == 1 and move_direction == "w":
+			return 0
 
 	except IndexError:
 		return 0
@@ -693,40 +733,153 @@ def interpret_lines(temp_array):
 			continue
 		#allow the \ line-continuation character.
 		while temp_array[cur_line][-1:] == "\\":  #really a single backslash, BTW.
-			temp_array[cur_line] = temp_array[cur_line][:-1] + temp_array[cur_line+1][:-1]
+			temp_array[cur_line] = temp_array[cur_line][:-1] + temp_array[cur_line+1][:-1].strip()
 			temp_array.pop(cur_line+1)
 			temp_array[cur_line].strip()
 		cur_line += 1
 	return temp_array
 
+
+sounds = {}
+nosound = 0
+#This loads the sounds
+def load_sounds():
+	global sounds
+	global nosound
+	sounds =  {}
+	if nosound == 1: return 0
+	try:
+		pygame.mixer.init()
+	except:
+		print "Unable to init sound."
+		nosound = 1
+		return 0
+
+	for root, dirs, files in walk(mod_directory + "/sound"):
+		(head, tail) = path.split(root)
+		if (tail != "CVS"):
+			for soundname in files:
+				#if image is in a sub-dir:
+				tmp_name = soundname[:-5]+soundname[-4:]
+				tmp_number = int(soundname[-5])
+				if (root != mod_directory + "/sound"):
+					i = len(mod_directory + "/sound")
+					base_name = root[i:] + "/" + tmp_name
+				else: #if image is in root dir
+					base_name = tmp_name
+				if not sounds.has_key(base_name): sounds[base_name] = {}
+				sounds[base_name][tmp_number] = \
+					pygame.mixer.Sound(root + "/" + soundname)
+
+def play_sound(sound_name):
+	if not sounds.has_key(sound_name):
+		print "missing sound set "+sound_name
+	dict_size = len(sounds[sound_name])
+
+	sounds[sound_name][int(random() * dict_size)].play()
+
+
+#create the fonts needed.
+font = pygame.font.Font(None, 14)
+
+#colors:
+colors = {}
+
+def fill_colors():
+	colors["white"] = (255, 255, 255, 255)
+	colors["black"] = (0, 0, 0, 255)
+	colors["hp_red"] = (238, 5, 5, 255)
+	colors["hp_green"] = (5, 187, 5, 255)
+	colors["ep_blue"] = (5, 5, 238, 255)
+	colors["dark_red"] = (125, 0, 0, 255)
+	colors["dark_green"] = (122, 169, 110, 255)
+	colors["dark_blue"] = (0, 0, 125, 255)
+	colors["light_red"] = (255, 50, 50, 255)
+	colors["light_green"] = (50, 255, 50, 255)
+	colors["light_blue"] = (50, 50, 255, 255)
+	colors["purple"] = (96, 96, 144, 255)
+	colors["light_gray"] = (227, 227, 227, 255)
+	colors["very_dark_blue"] = (32, 32, 47, 255)
+	colors["dh_green"] = (185, 229, 173, 255)
+fill_colors()
+
 #given a directory relative to g.mod_directory, will return a dictionary
 #of all images in that directory, and all subdirectories.
 def read_images(dir_name):
-	image_dictionary =  {"blank" : ImageTk.PhotoImage("RGB", (32, 32))}
-	for root, dirs, files in walk("../modules/default/" + dir_name):
-		(head, tail) = path.split(root)
-		if (tail != "CVS"):
-			for tilename in files:
-				#if image is in a sub-dir:
-				if (root != "../modules/default/" + dir_name):
-					i = len("../modules/default/" + dir_name)
-					image_dictionary[root[i:] + "/" + tilename] = \
-						ImageTk.PhotoImage(file=root + "/" + tilename)
-				else: #if image is in root dir
-					image_dictionary[tilename] = \
-						ImageTk.PhotoImage(file=root + "/" + tilename)
+	if pygame.image.get_extended() == 0:
+		print "Error: SDL_image required. Exiting."
+		sys.exit()
+	image_dictionary =  {"blank" : pygame.Surface((32, 32))}
+	image_dictionary = inner_read_images("../modules/default/" + dir_name,
+			image_dictionary)
+	image_dictionary = inner_read_images(g.mod_directory + dir_name,
+			image_dictionary)
 
-	for root, dirs, files in walk(g.mod_directory + dir_name):
-		(head, tail) = path.split(root)
-		if (tail != "CVS"):
-			for tilename in files:
-				#if image is in a sub-dir:
-				if (root != g.mod_directory + dir_name):
-					i = len(g.mod_directory + dir_name)
-					image_dictionary[root[i:] + "/" + tilename] = \
-						ImageTk.PhotoImage(file=root + "/" + tilename)
-				else: #if image is in root dir
-					image_dictionary[tilename] = \
-						ImageTk.PhotoImage(file=root + "/" + tilename)
 	return image_dictionary
 
+
+def inner_read_images(dir_name, image_dictionary):
+	for root, dirs, files in walk(dir_name):
+		(head, tail) = path.split(root)
+		if (tail != "CVS"):
+			for tilename in files:
+				#if image is in a sub-dir:
+				if (root != dir_name):
+					i = len(dir_name)
+					image_dictionary[root[i:] + "/" + tilename] = \
+						pygame.image.load(root + "/" + tilename).convert_alpha()
+				else: #if image is in root dir
+					image_dictionary[tilename] = \
+						pygame.image.load(root + "/" + tilename).convert_alpha()
+	return image_dictionary
+
+#creates a box, as used throughout the game.
+def create_norm_box(xy, size, outline_color="black", inner_color="light_gray"):
+	screen.fill(colors[outline_color],
+		(xy[0], xy[1], size[0], size[1]))
+	screen.fill(colors[inner_color],
+		(xy[0]+1, xy[1]+1, size[0]-2, size[1]-2))
+
+#given a surface, string, font, char to underline (int; -1 to len(string)),
+#xy coord, and color, print the string to the surface.
+#Align (0=left, 1=Center, 2=Right) changes the alignment of the text
+def print_string(surface, string_to_print, font, xy, color=colors["black"],
+		align=0, width=-1):
+	string_to_print = string_to_print.replace("\t", "     ")
+	if align != 0:
+		temp_size = font.size(string_to_print)
+		if align == 1: xy = (xy[0] - temp_size[0]/2, xy[1])
+		elif align == 2: xy = (xy[0] - temp_size[0], xy[1])
+	temp_text = font.render(string_to_print, 1, color)
+	if width != -1:
+		surface.blit(temp_text, xy, (0, 0, width, temp_text.get_size()[1]))
+	else:
+		surface.blit(temp_text, xy)
+
+#Used to display descriptions and such. Automatically wraps the text to fit
+#within a certain width.
+#Note that \n can be used for newlines, but it must be used as
+#line1 \\n line2 in code, (separated by spaces, with the \ escaped), or as
+#line1 \n line2 in scripts.
+def print_multiline(surface, string_to_print, font, width, xy, color="black"):
+	string_to_print = string_to_print.replace("\t", "     ")
+	start_xy = xy
+	string_array = string_to_print.split(" ")
+
+	num_of_lines = 1
+	for string in string_array:
+		string += " "
+		temp_size = font.size(string)
+
+		if string == "\n ":
+			num_of_lines += 1
+			xy = (start_xy[0], xy[1]+temp_size[1])
+			continue
+		temp_text = font.render(string, 1, colors[color])
+
+		if (xy[0]-start_xy[0])+temp_size[0] > width:
+			num_of_lines += 1
+			xy = (start_xy[0], xy[1]+temp_size[1])
+		surface.blit(temp_text, xy)
+		xy = (xy[0]+temp_size[0], xy[1])
+	return num_of_lines
