@@ -89,8 +89,9 @@ item = []
 #takes an item name, and returns its location in the
 #item[] array, with -1 == nonexisting.
 def finditem(name):
+	tmpname = name.lower()
 	for i in range(len(item)):
-		if name.lower() == item[i].name.lower():
+		if tmpname == item[i].name.lower():
 			return i
 	return -1
 
