@@ -46,7 +46,8 @@ class player_class:
 		self.cur_hero = "people/hero_w.png"
 
 		#skills array. Each skill is a separate line in the array. Each line goes:
-		#name, effect, level, price, description, acquired
+		#name, effect, level, price, description, acquired, scripting (if any),
+		#then picture.
 		#effect is an integer that tells battle.py which case in a select to pick.
 		#level is the skillpoints required to get the skill,
 		#price is the ep needed to use.
@@ -68,10 +69,21 @@ class player_class:
 		self.exp = int(self.exp) + int(input_exp)
 		if 1 > self.exp: self.exp = 0
 		#Has the player gained a level?
-		if int(self.exp) >= int(10*(self.level + 1)*(self.level + 1)):
+		if self.exp_till_level() <= 0:
 			self.level=self.level+1
 			g.main.print_message("You gain a level.")
-			g.action.activate_lines(g.xgrid, g.ygrid, g.zgrid, g.levelup_act)
+			g.action.activate_lines(g.xgrid, g.ygrid, g.zgrid,
+					g.levelup_act)
+
+	def exp_till_level(self):
+		if g.exp_list == "":
+			return_val = int(10*(self.level+1)*(self.level+1)) - int(self.exp)
+		else:
+			if len(g.exp_list) > self.level:
+				return_val = int(g.exp_list[self.level]) - int(self.exp)
+			else: return_val = 9999
+		if return_val < 0: return_val = 0
+		return return_val
 
 	#function for permanently changing the stats of a player.
 	#stat=hp, ep, (adj_)maxhp, (adj_)maxep, (adj_)attack, (adj_)defense,
